import os
import socket
import appdirs
import validators
import sys


from mediaDB.common import wget, get_date, get_current_time

DEBUG_MODE_ENABLE = True
hostname = socket.gethostname()
IP = socket.gethostbyname(hostname)
APP_NAME = "mediaDB"
APP_AUTHOR = "Strange500"

VAR_DIR = appdirs.user_cache_dir(appname=APP_NAME, appauthor=APP_AUTHOR)
CONF_DIR = appdirs.user_config_dir(appname=APP_NAME, appauthor=APP_AUTHOR)
SETTINGS_DIR = os.path.join(CONF_DIR, "setting")


GENERAL_SETTINGS_FILE, __GENERAL_SETTINGS_URL = os.path.join(SETTINGS_DIR, "COMMON"),""
TMDB_MOVIE_BAN_FILE = os.path.join(SETTINGS_DIR, "banned_movies.list")
TMDB_TV_BAN_FILE = os.path.join(SETTINGS_DIR, "banned_tv.list")
INDEXERS_FILE, __INDEXERS_URL = os.path.join(SETTINGS_DIR,"Indexers.json"), ""
METADONNEE_PROVIDERS_FILE, __METADONNEE_PROVIDERS_URL = os.path.join(SETTINGS_DIR, "MetaProviders.json"), ""

PROVIDERS_LIST = {}
os.makedirs(VAR_DIR, exist_ok=True)
os.makedirs(CONF_DIR, exist_ok=True)
os.makedirs(SETTINGS_DIR, exist_ok=True)

for couple in [(GENERAL_SETTINGS_FILE, __GENERAL_SETTINGS_URL), (TMDB_MOVIE_BAN_FILE, ""), (TMDB_TV_BAN_FILE, ""),
                  (INDEXERS_FILE, __INDEXERS_URL), (METADONNEE_PROVIDERS_FILE, __METADONNEE_PROVIDERS_URL)]:
    file, url = couple
    #print(file)
    #print(url)
    if not os.path.isfile(file) and validators.url(url):
        wget(url, file)
    else:
        with open(file, "w") as f:
            f.write("")

def log(to_log: str, info: bool|None=False, warning:bool|None=False, error:bool|None=False, debug:bool|None=False) -> None:
    sentence = f"[{get_date()}] [{get_current_time}] - "
    if debug and DEBUG_MODE_ENABLE:
        sentence += f"DEBUG : {to_log}"
        sys.stdout.write(sentence)
        print(sentence)
    elif warning:
        sentence += f"WARNING : {to_log}"
        sys.stderr.write(sentence)
    elif error:
        sentence += f"ERROR : {to_log}"
        sys.stderr.write(sentence)
    elif info:
        sentence += f"{to_log}"
        sys.stdout.write(sentence)
    return 



from mediaDB.exceptions import *
from mediaDB.common import *
from mediaDB.settings import *
from mediaDB.mediaTypes import mediaType
class ProviderCommon():

    # VARIABLES
    SETTING_DIRECTORY = os.path.join(CONF_DIR, "Providers")
    VAR_DIRECTORY = os.path.join(VAR_DIR, "Providers")
    PROXY = FlareSolverrProxy

    # CREATING FILES & DIRECTORIES
    os.makedirs(SETTING_DIRECTORY, exist_ok=True)
    os.makedirs(VAR_DIRECTORY, exist_ok=True)
    

    def make_result(tmdb_id:int, media_type:int, release_date:str, last_air_date:str | None = None, adult: bool | None = None, genres: list[int] | None = None, in_production: bool | None = False, last_episode_to_air: dict | None = None, title: str | None = None, other_titles:list[str] | None = None, next_episode_to_air: dict | None = None, number_of_episodes:int | None = None, number_of_season: int | None = None, original_language:str | None = None, seasons: dict | None = None, status: str | None = None, info_date: str|None= None, **kwargs) -> dict:
        """
        Normalize data, in order to be used by metaProvider.py
        {
            "adult": bool,
            "release_date": string,
            "last_air_date": string,
            "genres": [int],
            "tmdb_id": int,
            "in_production": bool,
            "last_episode_to_air": {
                "air_date" : string,
                "episode_number": int,
                "season_number": int
            },
            "title": string,
            "other_titles": [string],
            "next_episode_to_air": {
                "air_date" : string,
                "episode_number": int,
                "season_number": int
            },
            "number_of_episodes": int,
            "number_of_season": int,
            "media_type": int,
            "original_language": string,
            "seasons": {
                "1": {
                    "air_date": string,
                    "episode_count": int,
                    "name": string,
                    "episodes_list": [int]
                    }
                },
            "status": string,

        }"""
        if adult is None:
            adult = False

        if release_date is None or not isinstance(release_date, str) or not is_date_valid(release_date):
            print(release_date)
            raise ValueError("method make_result: release date cannot be None | release date need to be formatted as '%Y-%m-%d")

        if last_air_date is None:
            last_air_date = release_date

        if not isinstance(last_air_date, str) or not is_date_valid(last_air_date):
            raise ValueError("method make_result: last_air_date need to be formated as '%Y-%m-%d")
    
        if genres is None:
            genres = []

        if not isinstance(genres, list) or not itemsAreType(genres, int): 
            raise ValueError("method make_result: genre has to be list of ints")
        
        for ids in genres:
            if not isinstance(ids, int):
                raise ValueError("method make_result: genre ids must be int")
            
        if not isinstance(tmdb_id, int):
            raise ValueError("method make_results: tmdb_id has to be int")

        if not isinstance(in_production, bool):
            raise ValueError("method make_results: in_production has to be bool")
        
        if not isinstance(last_episode_to_air, dict) and not last_episode_to_air is None:
            print(isinstance(last_episode_to_air, dict))
            raise ValueError("method make_results: last_episode_to_air has to be dict or None")
        
        if not isinstance(title, str):
            raise ValueError("method make_results: title has to be str")
        
        if not isinstance(other_titles, list) or not itemsAreType(other_titles, str):
            raise ValueError("method make_results: other_titles has to be list and items are str")
        
        if (not isinstance(number_of_episodes, int) or number_of_episodes < 0) and number_of_episodes is not None:
            raise ValueError("method make_results: number_of_episodes has to be int and > 0")
        
        if (not isinstance(number_of_season, int) or number_of_season < 0) and number_of_season is not None:
            raise ValueError("method make_results: number_of_seasons has to be int and > 0")
        try:
            mediaType(media_type)
        except MediaTypeDoesNotExist:
            raise ValueError("method make_results: media_type has to be int and exist for MediaType class")
        
        if not isinstance(original_language, str):
            raise ValueError("method make_results: original_language has to be str")
        
        if not isinstance(seasons, dict) and not seasons is None:
            raise ValueError("method make_results: seasons has to be list or None")
        
        if not isinstance(status, str):
            raise ValueError("method make_results: status has to be str")
        
        if not isinstance(info_date, str) or not is_date_valid(info_date):
            raise ValueError("method make_results: info_date has to be str and valid date (%Y-%m-%d)")
        
        
        result = {
            "adult": adult,
            "release_date": release_date,
            "last_air_date": release_date,
            "genres": genres,
            "tmdb_id": tmdb_id,
            "in_production": in_production,
            "last_episode_to_air": last_episode_to_air,
            "title": title,
            "other_titles": other_titles,
            "next_episode_to_air": next_episode_to_air,
            "number_of_episodes": number_of_episodes,
            "number_of_season": number_of_season,
            "media_type": media_type,
            "original_language": original_language,
            "seasons": seasons,
            "status": status,
            "info_date": info_date

        }
        return result


    def checkConfig(config: dict, keys: dict) -> bool:
        for key in keys:
            if config.get(key, None) is not None:
                if isinstance(config.get(key), dict) and isinstance(keys.get(key, None), dict) and not ProviderCommon.checkConfig(config[key], keys[key]):
                    return False
                continue
            else:
                return False
        return True

    



from mediaDB.common import *
from mediaDB.settings import *
from mediaDB.Database import *
from mediaDB.flaresolver import *
from mediaDB.indexer import *
from mediaDB.mediaTypes import *
from mediaDB.metaProviders import *

from flask import Flask, jsonify, request, abort
from flask_cors import cross_origin

from json import load, dump

app = Flask(__name__)

######## METAPROVIDER ##########

@app.route("/provider/tv/get_info", methods=["POST"])
@cross_origin()
def getTVInfo():
    provider = request.form.get("provider")
    id = request.form.get("id")
    title = request.form.get("title")
    media_type = 3
    if not (isinstance(provider, str) and (canConvertInt(id) or isinstance(title, str))):
        abort(400)
    if PROVIDERS_LIST.get(provider) is None:
        abort(400)
    p = MetaProviders(provider, PROVIDERS_LIST.get(provider), media_type)
    if id is not None:
        return jsonify(p.getMediaData(tmdb_id=int(id)))
    elif title is not None:
        return jsonify(p.getMediaData(title=title))
    return "ERROR"

@app.route("/provider/tv/exist", methods=["POST"])
@cross_origin()
def TVExist():
    provider = request.form.get("provider")
    id = request.form.get("id")
    title = request.form.get("title")
    media_type = 3
    if not (isinstance(provider, str) and (canConvertInt(id) or isinstance(title, str))):
        abort(400)
    if PROVIDERS_LIST.get(provider) is None:
        abort(400)
    p = MetaProviders(provider, PROVIDERS_LIST.get(provider), media_type)
    if id is not None:
        return jsonify(p.mediaExistbyId(int(id)))
    elif title is not None:
        return jsonify(p.mediaExistbyName(title=title))
    return "ERROR"


@app.route("/provider/tv/season/get_info", methods=["POST"])
@cross_origin()
def getTVSeasonInfo():
    provider = request.form.get("provider")
    id = request.form.get("id")
    title = request.form.get("title")
    season = request.form.get("season")
    media_type = 3
    if not (isinstance(provider, str) and (canConvertInt(id) or isinstance(title, str))\
            and canConvertInt(season)):
        abort(400)
    if PROVIDERS_LIST.get(provider) is None:
        abort(400)
    p = MetaProviders(provider, PROVIDERS_LIST.get(provider), media_type)
    if id is not None:
        return jsonify(p.getSeasonInfo(tmdb_id=int(id), season=season))
    elif title is not None:
        return jsonify(p.getSeasonInfo(title=title, season=season))
    return "ERROR"

@app.route("/provider/tv/season/exist", methods=["POST"])
@cross_origin()
def TVSeasonExist():
    provider = request.form.get("provider")
    id = request.form.get("id")
    title = request.form.get("title")
    season = request.form.get("season")
    media_type = 3
    if not (isinstance(provider, str) and (canConvertInt(id) or isinstance(title, str))\
            and canConvertInt(season)):
        abort(400)
    if PROVIDERS_LIST.get(provider) is None:
        abort(400)
    p = MetaProviders(provider, PROVIDERS_LIST.get(provider), media_type)
    if id is not None:
        return jsonify(p.seasonExistbyId(id=id, season=season))
    elif title is not None:
        return jsonify(p.seasonExistbyName(title=title, season=season))
    return "ERROR"

@app.route("/provider/tv/episode/exist", methods=["POST"])
@cross_origin()
def TVEpisodeExist():
    provider = request.form.get("provider")
    id = request.form.get("id")
    title = request.form.get("title")
    season = request.form.get("season")
    episode = request.form.get("episode")
    media_type = 3
    if not (isinstance(provider, str) and (canConvertInt(id) or isinstance(title, str))\
            and canConvertInt(season) and canConvertInt(episode)):
        abort(400)
    if PROVIDERS_LIST.get(provider) is None:
        abort(400)
    p = MetaProviders(provider, PROVIDERS_LIST.get(provider), media_type)
    if id is not None:
        return jsonify(p.episodeExistbyId(id=id, season=season, episode=episode))
    elif title is not None:
        return jsonify(p.episodeExistbyName(title=title, season=season, episode=episode))
    return "ERROR"



@app.route("/provider/movie/get_info", methods=["POST"])
@cross_origin()
def getMovieInfo():
    provider = request.form.get("provider")
    id = request.form.get("id")
    title = request.form.get("title")
    media_type = 1
    if not (isinstance(provider, str) and (canConvertInt(id) or isinstance(title, str))):
        abort(400)
    if PROVIDERS_LIST.get(provider) is None:
        abort(400)
    p = MetaProviders(provider, PROVIDERS_LIST.get(provider), media_type)
    if id is not None:
        return jsonify(p.getMediaData(tmdb_id=int(id)))
    elif title is not None:
        return jsonify(p.getMediaData(title=title))
    return "ERROR"

@app.route("/provider/movie/exist", methods=["POST"])
@cross_origin()
def MovieExist():
    provider = request.form.get("provider")
    id = request.form.get("id")
    title = request.form.get("title")
    media_type = 1
    if not (isinstance(provider, str) and (canConvertInt(id) or isinstance(title, str))):
        abort(400)
    if PROVIDERS_LIST.get(provider) is None:
        abort(400)
    p = MetaProviders(provider, PROVIDERS_LIST.get(provider), media_type)
    if id is not None:
        return jsonify(p.mediaExistbyId(int(id)))
    elif title is not None:
        return jsonify(p.mediaExistbyName(title=title))
    return "ERROR"





class MediaNotFoundERROR(Exception):
    "Raise if an indexer can't find a media in it's database"

class MediaTypeNotSupported(Exception):
    "Raise if media type is not supported"

class MediaTypeDoesNotExist(Exception):
    "Raised if the media type is not initialsed"

class ProviderConfigError(Exception):
    "Raised if a config file is not initialised"

class MediaTypesFilesDoesNotExist(Exception):
    "Raised if a config file is not initialised"

class ProviderConfigError(Exception):
    "Raised if a config file is not initialised"

class TMDBError(Exception):
    "Raised if a error occured in tmdb manipulator"

class IdDoesNotExist(TMDBError):
    "Raised if you try to acess an tmdb id that doesn't exist"

class MalformedTMDBInfo(TMDBError):
    "Raised is tmdb_info does not have the required keys"


from mediaDB.common import *
from mediaDB.settings import *
from mediaDB.exceptions import *
from json import load


class mediaType():

    MEDIA_TYPES_FILE = os.path.join(CONF_DIR, "media_types.json")
    MEDIA_TYPES_JSON_URL = "https://raw.githubusercontent.com/Strange500/mediaDB/main/exemples/media_types.json"

    if not os.path.isfile(MEDIA_TYPES_FILE) and not wget(MEDIA_TYPES_JSON_URL, MEDIA_TYPES_FILE):
        raise MediaTypesFilesDoesNotExist
    with open(MEDIA_TYPES_FILE, "r") as f:
        MEDIA_TYPES = load(f)
    def __init__(self, id:int) -> None:
        try:
            data = self.MEDIA_TYPES[f"{id}"]
        except KeyError:
            raise MediaTypeDoesNotExist
        self.__name = data["name"]
        self.__id = id
        self.have_season = data["have_season"]
        self.only_sound = data["only_sound"]
        self.readable = data["readable"]
        




# from mediaDB.common import *
# from mediaDB.settings import *
# from mediaDB.extension.Indexers.common import IndexerCommon

class Nyaa_manipulator():
    ...
#     NAME:str
#     SETTING_FILE: str
#     CONFIG: dict
#     CACHE_DB_ANIME: dict
#     with alive_bar(9, manual=True, title='Initializing tmdb provider', **bar_setting) as bar:

#         # CONST
#         __DATE = datetime.now().strftime("%m_%d_%Y")
#         NAME = "TMDB"
#         CONFIG_EXEMPLE_URL = "https://raw.githubusercontent.com/Strange500/mediaDB/main/exemples/TMDB"
#         SETTING_FILE = os.path.join(ProviderCommon.SETTING_DIRECTORY, NAME)
#         VAR_DIRECTORY = os.path.join(ProviderCommon.VAR_DIRECTORY, NAME)
#         CACHE_DIRECTORY = os.path.join(VAR_DIRECTORY, "cache")
#         GENRE_MOVIE_FILE = os.path.join(CACHE_DIRECTORY, "genre_movie.json")
#         GENRE_TV_FILE = os.path.join(CACHE_DIRECTORY, "genre_tv.json")
#         TMDB_EXPORT_URL = "https://files.tmdb.org/p/exports/"
#         IDS_TV_FILE = os.path.join(CACHE_DIRECTORY, f"tv_ids_{__DATE}.json")
#         IDS_TV_URL = f"{TMDB_EXPORT_URL}tv_series_ids_{__DATE}.json.gz"
#         IDS_MOVIE_FILE = os.path.join(CACHE_DIRECTORY, f"movies_ids_{__DATE}.json")
#         IDS_MOVIES_URL = f"{TMDB_EXPORT_URL}movie_ids_{__DATE}.json.gz"
#         CACHE_DB_TV_FILE = os.path.join(CACHE_DIRECTORY, "DB_tv.json")
#         CACHE_DB_MOVIE_FILE = os.path.join(CACHE_DIRECTORY, "DB_movie.json")
#         media_types = [1, 3]

#         # CREATE NEEDED FILES & DIRECTORY
#         os.makedirs(VAR_DIRECTORY, exist_ok=True)
#         os.makedirs(CACHE_DIRECTORY, exist_ok=True)
#                 # Download TMDB config file if not created
#         if not isfile(SETTING_FILE) and not wget(CONFIG_EXEMPLE_URL, SETTING_FILE):
#             raise ProviderConfigError
#         if not isfile(CACHE_DB_TV_FILE) :
#             with open(CACHE_DB_TV_FILE, "w", encoding="utf-8") as f:
#                 dump({}, f, indent=5)
#         if not isfile(CACHE_DB_MOVIE_FILE) :
#             with open(CACHE_DB_MOVIE_FILE, "w", encoding="utf-8") as f:
#                 dump({}, f, indent=5)
#         # SETTING UP 
#         CONFIG = parseConfig(SETTING_FILE)
#         if not ProviderCommon.checkConfig(CONFIG, {"api_key": 1, "timeout": 1}):
#             raise ProviderConfigError
#         API_KEY = CONFIG["api_key"]
#         tmdb.API_KEY = API_KEY[0]
#         tmdb.REQUESTS_TIMEOUT = int(CONFIG["timeout"][0])
#         bar(0.001)
#             # Download tmdb ids file
#         if not isfile(GENRE_MOVIE_FILE):
#             movie_list = tmdb.Genres().movie_list()
#             with open(GENRE_MOVIE_FILE, "w") as f:
#                 save_json(f, movie_list)
#         bar(0.03)
#         if not isfile(GENRE_TV_FILE):
#             tv_list = tmdb.Genres().tv_list()
#             with open(GENRE_TV_FILE, "w") as f:
#                 save_json(f, tv_list)
#         bar(0.03)
#         with open(GENRE_MOVIE_FILE, "r") as f:
#             MOVIE_GENRE_IDS = load(f)
#         with open(GENRE_TV_FILE, "r") as f:
#             TV_GENRE_IDS = load(f)
#         bar(0.001)
#             # update ids files
#         if not isfile(IDS_MOVIE_FILE) :

#             if wget(IDS_MOVIES_URL, IDS_MOVIE_FILE+".gz"):
#                 gzExtract(IDS_MOVIE_FILE+".gz", IDS_MOVIE_FILE) 
#                 makeIdsFile(IDS_MOVIE_FILE)
#         bar(0.59)
#         if not isfile(IDS_TV_FILE) :
#             if wget(IDS_TV_URL, IDS_TV_FILE+".gz"):
#                 gzExtract(IDS_TV_FILE+".gz", IDS_TV_FILE)    
#                 makeIdsFile(IDS_TV_FILE)
#         bar(0.2)
#             # loads ids
#         with open(IDS_MOVIE_FILE, "r", encoding="utf-8") as f:
#             IDS_MOVIE = load(f)
#         bar(0.05)
#         with open(IDS_TV_FILE, "r", encoding="utf-8") as f:
#             IDS_TV = load(f)
#         bar(0.009)
#             # loads DB
#         with open(CACHE_DB_TV_FILE, "r", encoding="utf-8") as f:
#             CACHE_DB_TV = load(f)
#         with open(CACHE_DB_MOVIE_FILE, "r", encoding="utf-8") as f:
#             CACHE_DB_MOVIE = load(f)
#         bar(1.)


from mediaDB.API import app
from mediaDB.common import alive_bar, forbidden_car, is_video, is_connected,key_value_in_dic_key, make_response_api, next_id, parseConfig, is_date_valid, save_json
from mediaDB.Database import Database
from mediaDB.exceptions import MediaNotFoundERROR, MediaTypeNotSupported, MediaTypeDoesNotExist
from mediaDB.flaresolver import FlareSolverrProxy
from mediaDB.indexer import indexer
from mediaDB.mediaTypes import mediaType
from mediaDB.metaProviders import MetaProviders
from mediaDB.extension import Providers, Indexers, Parser
from mediaDB.settings import DEBUG_MODE_ENABLE, hostname, IP, APP_AUTHOR, APP_NAME, VAR_DIR, CONF_DIR, SETTINGS_DIR, GENERAL_SETTINGS_FILE, TMDB_MOVIE_BAN_FILE, TMDB_TV_BAN_FILE, INDEXERS_FILE, METADONNEE_PROVIDERS_FILE





import os
import socket
import appdirs
from urllib.parse import urlparse
import validators
import gzip
import shutil
from copy import deepcopy
import requests
import re
import sys
from json import dump, load, loads, JSONDecodeError
from datetime import datetime
from alive_progress import alive_bar
# local imports
from mediaDB.flaresolver import FlareSolverrProxy

bar_setting = {"bar" :"classic2", "spinner":"pulse"}

def forbidden_car(name):
    """
    Removes forbidden characters from a file name.

    Args:
        name (str): The file name to be processed.

    Returns:
        str: The processed file name with forbidden characters removed.

    Example:
        >>> forbidden_car("file?name")
        'filename'
    """
    for car in ["?", '"', "/", "\\", "*", ":", "<", ">", "|"]:
        name = name.replace(car, "")
    return name

def is_video(file_path):
    """
    Checks if a file at the given path is a video file.

    Args:
        file_path (str): The path to the file.

    Returns:
        bool: True if the file is a video file, False otherwise.
    """
    video_extensions = ['.mp4', '.avi', '.mov', '.wmv', '.mkv']
    file_ext = os.path.splitext(file_path)[1].lower()
    return file_ext in video_extensions

def is_connected() -> bool:
    try:
        requests.get("https://google.com")
    except requests.exceptions.ConnectionError:
        return False
    return True

def key_value_in_dic_key(dic: dict, key: str, value) -> bool:
    for ids in dic:
        val = dic[ids].get(key, None)
        if val == value:
            return True
    return False

def make_response_api(status : bool, detail: str):
    response = "ok"
    if not status:
        response = "failed"
    return {"status": response,
            "detail": detail}

def next_id(dic: dict) -> int:
    max_id = -1
    for key in dic:
        key = str(key)
        if key.isnumeric() and max_id < int(key):
            max_id = int(key)
    return max_id + 1

def parseConfig(file_path) -> dict:
        """bonjour"""
        try:
            config = {}
            with open(file_path, "r", encoding="utf-8") as f:
                for line in f:
                    if line[0] in ["#", "\n", ""]:
                        continue
                    line = line.replace("\n", "").split(" = ")
                    if "," in line[1]:
                        line[1] = [elt.strip() for elt in line[1].split(",")]
                    else:
                        line[1] = [line[1].strip()]
                    arg1, arg2 = line[0].strip(), line[1]
                    config[arg1] = arg2

            return config
        except IOError:
            return 
        

def is_date_valid(date_string, format='%Y-%m-%d'):
    try:
        datetime.strptime(date_string, format)
        return True
    except ValueError:
        return False
    
def save_json(f, obj: dict):
    dump(obj, f, indent=5)


        
def create_config_file(file_path:str, content: str):
    with open(file_path, "w") as f:
        f.write(file_path)

def wget(url: str, save_path: str) -> bool:
    try:
        response = requests.get(url)
    except requests.RequestException:
        return False
    try:
        with open(save_path, "wb") as f:
            f.write(bytes(response.content))
    except IOError:
        return False
    return True

def gzExtract(gz_file:str, file_name:str):
    with gzip.open(gz_file, 'rb') as f_in:
        with open(file_name, 'wb') as f_out:
            shutil.copyfileobj(f_in, f_out)
    os.remove(gz_file)

def makeIdsFile(path:str):
    dic = {}
    with open(path, "r", encoding="utf-8") as f:
        for lines in f:
            try:
                js = loads(lines)
            except JSONDecodeError:
                continue
            if js.get("id", None) is None:
                continue
            if js.get("original_title", None) is None and js.get("original_name", None) is None:
                continue
            try:
                dic[js["id"]] = js["original_title"]
            except KeyError:
                dic[js["id"]] = js["original_name"]
    with open(path, "w") as f:
        dump(dic, f, indent=5)

def is_latin(chaine):
    motif = re.compile(r'[^a-zA-ZÀ-ÿ\s!@#$%^&*()_\-+=\[\]{};:\'",.<>/?\\|`~]+')
    return not motif.search(chaine)

def get_date(format: str|None = "%m_%d_%Y"):
    return datetime.now().strftime(format)

def itemsAreType(items: list, tp) -> bool:
    for item in items:
        if not isinstance(item, tp):
            return False
    return True


def get_current_time():
    current_time = datetime.now().strftime("%H:%M")
    return current_time

def update_progress_bar(msg:str, progress:int):
    bar_length = 20
    block = int(round(bar_length * progress))
    progress_str = f"\r[{block * '#' + (bar_length - block) * '-'}] {msg}"
    print(progress_str, end='', flush=True)

def canConvertInt(item):
    return (isinstance(item, str) and item.isnumeric()) or isinstance(item, int)

def replaceDots(string:str):
    return string.replace(".", " ")
def replaceUnderscore(string:str):
    return string.replace("_", " ")


from mediaDB.common import *
from mediaDB.settings import *
from mediaDB.exceptions import *
from thefuzz import process

class MetaProviders():
    """All MetaProviders must return metadata as the foolowing dictionnary :
    result = {
            "adult": bool,
            "release_date": string,
            "last_air_date": string,
            "genres": [int],
            "tmdb_id": int,
            "in_production": bool,
            "last_episode_to_air": {
                "air_date" : string,
                "episode_number": int,
                "season_number": int
            },
            "title": string,
            "other_titles": [string],
            "next_episode_to_air": {
                "air_date" : string,
                "episode_number": int,
                "season_number": int
            },
            "number_of_episodes": int,
            "number_of_season": int,
            "media_type": int,
            "original_language": string,
            "date_info": string,
            "seasons": {
                "1": {
                    "air_date": string,
                    "episode_count": int,
                    "name": string,
                    "episodes_list": [int]
                    }
                },
            "status": string,

        }"""

    def __init__(self, name: str, provider_manipulator : object, media_type : int) -> None:
        self.__name = name
        self.__manipulator = provider_manipulator()
        self.__media_types = provider_manipulator.media_types
        if not media_type in self.__media_types:
            raise MediaTypeNotSupported
        self.__media_type = media_type

    def __get_titles_info(self, results: list) -> dict:
        list_titles = {result["title"]: result for result in results}
        for info in results:
            for title in info["other_titles"]:
                list_titles[title] = info
        return list_titles

    def mediaExistbyName(self, title: str):
        if not isinstance(title, str):
            raise ValueError("method mediaExistByName: title has to be string")
        results = self.__manipulator.find(title, self.__media_type)
        list_titles = self.__get_titles_info(results)
        tuple_result = process.extractOne(title, [key for key in list_titles])
        if tuple_result is None:
            return False
        else :
            r, score = tuple_result
        if score > 90:
            return True
        else:
            return False
        
    def __mediaInfobyName(self, title: str):
        results = self.__manipulator.find(title, self.__media_type)
        list_titles = self.__get_titles_info(results)
        tuple_result = process.extractOne(title, [key for key in list_titles])
        if tuple_result is None:
            return None
        else :
            r, score = tuple_result
        if score > 90:
            return list_titles[r]
        else:
            return None
        

    def seasonExistbyName(self, title: str, season: int) -> bool:
        if self.mediaExistbyName(title):
            data = self.getMediaData(title=title)
            return season in [int(s) for s in data["seasons"]]
        else:
            return False

    def episodeExistbyName(self, title : str, season: int, episode: int) -> bool:
        if self.mediaExistbyName(title) and self.seasonExistbyName(title, season=season):
            data = self.getMediaData(title=title)
            s = dict(data["seasons"]).get(f"{season}", None)
            if s is None:
                return False
            else:
                return episode in s["episodes_list"]
            
    def mediaExistbyId(self, identifier: int):
        result = self.__manipulator.get(identifier, self.__media_type)
        return result is not None
        

    def seasonExistbyId(self, id: int, season: int) -> bool:
        if self.mediaExistbyId(id):
            data = self.getMediaData(tmdb_id=id)
            return season in [int(s) for s in data["seasons"]]
        else:
            return False

    def episodeExistbyId(self, id : int, season: int, episode: int) -> bool:
        if self.mediaExistbyId(id) and self.seasonExistbyId(id, season=season):
            data = self.getMediaData(tmdb_id=id)
            s = dict(data["seasons"]).get(f"{season}", None)
            if s is None:
                return False
            else:
                return episode in s["episodes_list"]


    def getMediaData(self, title:str|None = None, tmdb_id:int | None = None) -> dict:
        
        if (title is None and tmdb_id is None) or (title is not None and tmdb_id is not None):
            raise ValueError("methods getMediaData: you must use title or (xor) tmdb_id")
        if title is not None:
            with alive_bar( title=f"Getting data for '{title}'", **bar_setting) as bar:
                if not isinstance(title, str):
                    raise ValueError("methods getMediaData: title has to be instance of str")
                if not self.mediaExistbyName(title):
                    return None
                r= self.__mediaInfobyName(title)
                return r
        elif tmdb_id is not None:
            with alive_bar(title=f"Getting data for id: {tmdb_id}", **bar_setting) as bar:
                if not isinstance(tmdb_id, int):
                    raise ValueError("methods getMediaData: tmdb_id has to be instance of int")
                if not self.mediaExistbyId(tmdb_id):
                    return None
                r = self.__manipulator.get(tmdb_id, self.__media_type)
                return r
        
    def getSeasonInfo(self, season: int, title: str | None = None, tmdb_id: int | None = None) -> dict:
        if (title is None and tmdb_id is None) or (title is not None and tmdb_id is not None):
            raise ValueError("methods getSeasonInfo: you must use title or (xor) tmdb_id")
        if title is not None:
            if not isinstance(title, str):
                raise ValueError("methods getSeasonInfo: title has to be instance of str")
            if not self.seasonExistbyName(title, season):
                return None
            list_results = self.__manipulator.find(title, self.__media_type)
            list_titles = [result["title"] for result in list_results]
            title, score = process.extractOne(title, list_titles)
            if score > 90:
                media_info = list_results[list_titles.index(title)]
                return media_info["seasons"][f"{season}"]
            return None
        elif tmdb_id is not None:
            if not isinstance(tmdb_id, int):
                raise ValueError("methods getMediaData: tmdb_id has to be instance of int")
            if not self.seasonExistbyId(tmdb_id, season):
                return None
            return self.__manipulator.get(tmdb_id, self.__media_type)["seasons"][f"{season}"]
        
        


from requests import post, exceptions, Response
from requests import get as g
from json import loads



class FlareSolverrProxy():

    def __init__(self, domain : str,flareSolverrURL :str, timeout : int | None = 60000) -> None:
        self.__flaresolverrurl = flareSolverrURL
        self.__domain = domain
        self.__cookies = None
        self.__userAgent = None
        self.__timeout = timeout
        self.__data = {
            "cmd" : "request.get",
            "url" : self.__domain,
            "maxTimeout" : self.__timeout

        }

        self.get_cookies_and_userAgent()

    def get_cookies_and_userAgent(self):
        try:
            response = post(self.__flaresolverrurl, json=self.__data, headers={'Content-Type': 'application/json'})
        except (exceptions.RequestException) as e:
            self.__cookies = None
            self.__userAgent = None

        if response.status_code == 200:
            response_data = loads(response.content)
            cookies = response_data["solution"]["cookies"]
            self.__cookies = {cookie["name"] : cookie["value"] for cookie in cookies}

            self.__userAgent = response_data["solution"]["userAgent"]


        else:
            raise exceptions.ProxyError(response=response)

    def get(self, url : str) -> Response :
        args = {"url": url, 
                "cookies": self.__cookies, 
                "headers" : {"User-Agent": self.__userAgent}
            }
        
        response = g(**args)
        if response.status_code == 200:
            return response
        elif response.status_code == 403:
            self.get_cookies_and_userAgent()
            response = g(**args)
            return response
        else:
            raise exceptions.ProxyError(response=response)

        


from mediaDB.common import *
from mediaDB.exceptions import *
from mediaDB.settings import *
class indexer():

    def __init__(self, name : str, indexer_manipulator: object) -> None:
        self.__name = name
        self.__manipulator = indexer_manipulator
        self.media_types = indexer_manipulator.media_types
        self.__var_directory = os.path.join(VAR_DIR, "indexers", self.__name)
        self.__conf_directory = os.path.join(CONF_DIR, "indexers", self.__name)

        pass

    def get_ep(self, title: str, episode : int, season : int, media_type: int):
        result = self.__manipulator.search_ep(media_type=media_type)
        if result is None:
            raise MediaNotFoundERROR
        else:
            return result
        
    def get_batch(self, title: str, season : int, media_type: int):
        result = self.__manipulator.search_ep(media_type=media_type)
        if result is None:
            raise MediaNotFoundERROR
        else:
            return result
        
    def get_show(self, title: str, episode : int, season : int, media_type: int):
        result = self.__manipulator.search_ep(media_type=media_type)
        if result is None:
            raise MediaNotFoundERROR
        else:
            return result
        
        


import tmdbsimple as tmdb
from json import load
from os.path import isfile
from datetime import datetime
from thefuzz import process
from copy import deepcopy
# relative imports
from mediaDB.common import *
from mediaDB.mediaTypes import *
from mediaDB.extension.Providers.common import ProviderCommon
from mediaDB.settings import *
from about_time import about_time


class TMDB_manipulator(ProviderCommon):
    NAME:str
    SETTING_FILE: str
    CONFIG: dict
    API_KEY:str
    GENRE_MOVIE_FILE:str
    GENRE_TV_FILE:str
    IDS_MOVIE: dict
    IDS_TV: dict
    CACHE_DB_TV: dict
    CACHE_DB_MOVIE: dict
    with alive_bar(9, manual=True, title='Initializing tmdb provider', **bar_setting) as bar:

        # CONST
        __DATE = datetime.now().strftime("%m_%d_%Y")
        NAME = "TMDB"
        CONFIG_EXEMPLE_URL = "https://raw.githubusercontent.com/Strange500/mediaDB/main/exemples/TMDB"
        SETTING_FILE = os.path.join(ProviderCommon.SETTING_DIRECTORY, NAME)
        VAR_DIRECTORY = os.path.join(ProviderCommon.VAR_DIRECTORY, NAME)
        CACHE_DIRECTORY = os.path.join(VAR_DIRECTORY, "cache")
        GENRE_MOVIE_FILE = os.path.join(CACHE_DIRECTORY, "genre_movie.json")
        GENRE_TV_FILE = os.path.join(CACHE_DIRECTORY, "genre_tv.json")
        TMDB_EXPORT_URL = "https://files.tmdb.org/p/exports/"
        IDS_TV_FILE = os.path.join(CACHE_DIRECTORY, f"tv_ids_{__DATE}.json")
        IDS_TV_URL = f"{TMDB_EXPORT_URL}tv_series_ids_{__DATE}.json.gz"
        IDS_MOVIE_FILE = os.path.join(CACHE_DIRECTORY, f"movies_ids_{__DATE}.json")
        IDS_MOVIES_URL = f"{TMDB_EXPORT_URL}movie_ids_{__DATE}.json.gz"
        CACHE_DB_TV_FILE = os.path.join(CACHE_DIRECTORY, "DB_tv.json")
        CACHE_DB_MOVIE_FILE = os.path.join(CACHE_DIRECTORY, "DB_movie.json")
        media_types = [1, 3]

        # CREATE NEEDED FILES & DIRECTORY
        os.makedirs(VAR_DIRECTORY, exist_ok=True)
        os.makedirs(CACHE_DIRECTORY, exist_ok=True)
                # Download TMDB config file if not created
        if not isfile(SETTING_FILE) and not wget(CONFIG_EXEMPLE_URL, SETTING_FILE):
            raise ProviderConfigError
        if not isfile(CACHE_DB_TV_FILE) :
            with open(CACHE_DB_TV_FILE, "w", encoding="utf-8") as f:
                dump({}, f, indent=5)
        if not isfile(CACHE_DB_MOVIE_FILE) :
            with open(CACHE_DB_MOVIE_FILE, "w", encoding="utf-8") as f:
                dump({}, f, indent=5)
        # SETTING UP 
        CONFIG = parseConfig(SETTING_FILE)
        if not ProviderCommon.checkConfig(CONFIG, {"api_key": 1, "timeout": 1}):
            raise ProviderConfigError
        API_KEY = CONFIG["api_key"]
        tmdb.API_KEY = API_KEY[0]
        tmdb.REQUESTS_TIMEOUT = int(CONFIG["timeout"][0])
        bar(0.001)
            # Download tmdb ids file
        if not isfile(GENRE_MOVIE_FILE):
            movie_list = tmdb.Genres().movie_list()
            with open(GENRE_MOVIE_FILE, "w") as f:
                save_json(f, movie_list)
        bar(0.03)
        if not isfile(GENRE_TV_FILE):
            tv_list = tmdb.Genres().tv_list()
            with open(GENRE_TV_FILE, "w") as f:
                save_json(f, tv_list)
        bar(0.03)
        with open(GENRE_MOVIE_FILE, "r") as f:
            MOVIE_GENRE_IDS = load(f)
        with open(GENRE_TV_FILE, "r") as f:
            TV_GENRE_IDS = load(f)
        bar(0.001)
            # update ids files
        if not isfile(IDS_MOVIE_FILE) :

            if wget(IDS_MOVIES_URL, IDS_MOVIE_FILE+".gz"):
                gzExtract(IDS_MOVIE_FILE+".gz", IDS_MOVIE_FILE) 
                makeIdsFile(IDS_MOVIE_FILE)
        bar(0.59)
        if not isfile(IDS_TV_FILE) :
            if wget(IDS_TV_URL, IDS_TV_FILE+".gz"):
                gzExtract(IDS_TV_FILE+".gz", IDS_TV_FILE)    
                makeIdsFile(IDS_TV_FILE)
        bar(0.2)
            # loads ids
        with open(IDS_MOVIE_FILE, "r", encoding="utf-8") as f:
            IDS_MOVIE = load(f)
        bar(0.05)
        with open(IDS_TV_FILE, "r", encoding="utf-8") as f:
            IDS_TV = load(f)
        bar(0.009)
            # loads DB
        with open(CACHE_DB_TV_FILE, "r", encoding="utf-8") as f:
            CACHE_DB_TV = load(f)
        with open(CACHE_DB_MOVIE_FILE, "r", encoding="utf-8") as f:
            CACHE_DB_MOVIE = load(f)
        bar(1.)


    def genreExist(self, id:int, media_type:int):
        m, id_list = mediaType(media_type), None
        if m.have_season:
            id_list = self.TV_GENRE_IDS
        else:
            id_list = self.MOVIE_GENRE_IDS
        return id in [genre["id"] for genre in id_list["genres"]]
    
    def movieIdExist(self, id:int) -> bool:
        return self.IDS_MOVIE.get(f"{id}", None) is not None
    
    def tvIdExist(self, id:int) -> bool:
        return self.IDS_TV.get(f"{id}", None) is not None

    def getTitleTV(self, id: int) -> str:
        if self.tvIdExist(id):
            return self.IDS_TV[f"{id}"]
        
    def getTitleMovie(self, id: int) -> str:
        if self.movieIdExist(id):
            return self.IDS_MOVIE[f"{id}"]
        
    def __store_tmdb_tv_info(self, info_tmdb: dict):
        if (self.tvIdExist(info_tmdb["id"])):
            info = deepcopy(info_tmdb)
            id = info.pop("id")
            self.CACHE_DB_TV[f"{id}"] = info
            with open(self.CACHE_DB_TV_FILE, "w", encoding="utf-8") as f:
                dump(self.CACHE_DB_TV, f, indent=5, sort_keys=True)

    def __store_tmdb_movie_info(self, info_tmdb: dict):
        if (self.movieIdExist(info_tmdb["id"])):
            info = deepcopy(info_tmdb)
            id = info.pop("id")
            self.CACHE_DB_MOVIE[f"{id}"] = info
            with open(self.CACHE_DB_MOVIE_FILE, "w", encoding="utf-8") as f:
                dump(self.CACHE_DB_MOVIE, f, indent=5, sort_keys=True)

    # def __get_season_changes(self, changes_season: list[int], date:str)->dict:
    #     from pprint import pprint
    #     result = []
    #     for season_id in list(set(changes_season)):
    #         changes = tmdb.TV_Changes(season_id).season(start_date=date)["changes"]
    #         changes = [change["items"] for change in changes if change.get("items") is not None]
    #         changes = [change for change in changes if type(change) == list and len(change) > 0 and change[0]["value"].get("episode_id", None) is not None]
    #         if len(changes) > 0:
    #             changes = [item for change in changes for item in change]
    #         result = [*result, *[change["value"]["episode_id"] for change in changes if change.get("value", None) is not None and change["value"].get("episode_id") is not None ]]
            
    #     return 


    # def __get__changes_tv(self, info: dict) -> list[int] | None:
    #     date = info["info_date"]
    #     date = "2022-11-02"
    #     id = info["id"]
    #     result = []
    #     changes = tmdb.TV_Changes(id).series(start_date=date)["changes"]
    #     changes = [change["items"] for change in changes if change.get("items") is not None]
    #     changes = [change for change in changes if type(change) == list and len(change) > 0 and change[0]["value"].get("season_id", None) is not None]
    #     if len(changes) > 0:
    #         changes = [item for change in changes for item in change]
    #     for change in changes:
    #         value = change.get("value", None)
    #         if value is None:
    #             continue
    #         if dict(value).get("season_id", None) is not None:
    #             s_id = dict(value).get("season_id", None) 
    #             result.append(s_id)
    #     return self.__get_season_changes(result, date)

    def __get_info_from_cache_tv(self, id:int) -> dict|None:
        info = self.CACHE_DB_TV.get(f"{id}", None)
        if info is not None:
            date = datetime.strptime(info["info_date"], "%Y-%m-%d")
            if datetime.strptime(get_date(format="%Y-%m-%d"), "%Y-%m-%d")> date:
                return self.__getTVInfo(id)
            else:
                return info
        else:
            return info
        
    def __get_info_from_cache_movie(self, id:int) -> dict|None:
        info = self.CACHE_DB_MOVIE.get(f"{id}", None)
        if info is not None:
            date = datetime.strptime(info["info_date"], "%Y-%m-%d")
            if datetime.strptime(get_date(format="%Y-%m-%d"), "%Y-%m-%d")> date:                
                return self.__getMovieInfo(id)
            else:
                return info
        else:
            return info
        
    def findIdTV(self, title: str) -> int:
        titles = {self.IDS_TV[id]: id for id in self.IDS_TV}
        best, score = process.extractOne(title, [*titles])
        if score > 90:
            return int(titles[best])
        else:
            p = tmdb.Search()
            results_stat = p.tv(query=title)
            if results_stat["total_results"] > 0:
                return int(results_stat["results"][0]["id"])
            return -1
    
    def findIdMovie(self, title: str) -> int:
        titles = {self.IDS_MOVIE[id]: id for id in self.IDS_MOVIE}
        best, score = process.extractOne(title, [*titles])
        if score > 90:
            return int(titles[best])
        else:
            p = tmdb.Search()
            results_stat = p.movie(query=title)
            if results_stat["total_results"] > 0:
                return int(results_stat["results"][0]["id"])
            return -1
        
    def findTVByTitle(self, title: str) -> list[int]:
        return [info["id"] for info in tmdb.Search().tv(query=title)["results"] if info.get("id", None) is not None]
    
    def findMovieByTitle(self,title: str) -> list[int]:
        #print(tmdb.Search().movie(query=title) )
        return [info["id"] for info in tmdb.Search().movie(query=title)["results"] if info.get("id", None) is not None]
        
    def __make_seasons(self, info_tmdb:dict):
        id = info_tmdb["id"]
        if not self.tvIdExist(id):
            raise IdDoesNotExist(f"method __make_seasons: {id} does not exist")
        episode_group_id = [i for i in tmdb.TV(id).episode_groups()["results"] if i["name"] == "Seasons"]
        if episode_group_id == []:
            new_dic = {"seasons" : {}}
            season_data = info_tmdb["seasons"]
            for seasons in range(len(info_tmdb["seasons"])):
                air_date = season_data[seasons]["air_date"]
                episode_count = season_data[seasons]["episode_count"]
                episodes = [i for i in range(1, episode_count+1)]
                name = season_data[seasons]["name"]
                season_number = season_data[seasons]["season_number"]
                new_dic["seasons"][f"{season_number}"] = {
                            "air_date": air_date,
                            "episode_count": episode_count,
                            "name": name,
                            "episodes_list": episodes
                    }
            info_tmdb["number_of_season"] = len(new_dic["seasons"])
            info_tmdb["seasons"] = new_dic["seasons"]
            return info_tmdb
        episode_group_id = episode_group_id[0]["id"]
        new_dic = {"seasons" : {}}
        info = tmdb.TV_Episode_Groups(id=episode_group_id).info()['groups']
        for seasons in range(len(info)):
            air_date = info[seasons]['episodes'][0]["air_date"]
            episode_count = len(info[seasons]['episodes'])
            episodes = [ep_data["episode_number"] for ep_data in info[seasons]['episodes'] if ep_data.get("episode_number") is not None]
            name = info[seasons]["name"]
            season_number = info[seasons]["order"]
            new_dic["seasons"][f"{season_number}"] = {
                        "air_date": air_date,
                        "episode_count": episode_count,
                        "name": name,
                        "episodes_list": episodes
                }
        info_tmdb["number_of_season"] = len(info)
        info_tmdb["seasons"] = new_dic["seasons"]
        return info_tmdb
    
    def __alter_title_translations(self, info_tmdb: dict) -> dict:
        result = []
        tr = info_tmdb.get("translations", None)
        if tr is not None:
            try:
                tr = [trans["data"]["title"] for trans in tr["translations"] if trans["data"]["title"] != '' and is_latin(trans["data"]["title"]) ]
            except KeyError:
                tr = [trans["data"]["name"] for trans in tr["translations"] if trans["data"]["name"] != '' and is_latin(trans["data"]["name"]) ]
            for title in tr:
                result.append({"title": title})
        return result
            
    
    def __make_alter_titles(self, info_tmdb: dict) -> dict:
        if info_tmdb.get("media_type") == 3:
            alter = tmdb.tv.TV(info_tmdb['id']).alternative_titles()["results"]
            alter = [*alter, *self.__alter_title_translations(info_tmdb)]
        elif info_tmdb.get("media_type") == 1:
            alter = tmdb.movies.Movies(info_tmdb['id']).alternative_titles()["titles"]
            alter = [*alter, *self.__alter_title_translations(info_tmdb)]
        result = []
        for dic in alter:
            title = dic["title"]
            if is_latin(title) and title != '':
                result.append(title)
        if info_tmdb.get("name", None) is not None and is_latin(info_tmdb.get("name")) :
            result.append(info_tmdb.get("name", ""))
        if info_tmdb.get("original_name", None) is not None and is_latin(info_tmdb.get("original_name")) :
            result.append(info_tmdb.get("original_name", ""))
        info_tmdb["other_titles"] = list(set(result))
        return info_tmdb
        
    def __make_release_date(self, tmdb_info: dict) -> dict:
        if tmdb_info.get("release_date", None) is None and not tmdb_info.get("first_air_date", None) in [None, ""]:
            tmdb_info["release_date"] = tmdb_info["first_air_date"]
        elif (tmdb_info.get("release_date", None) in [None, ""] or not is_date_valid(tmdb_info.get("release_date", None))) and not tmdb_info.get("first_air_date", None) in [None, ""]:
            tmdb_info["release_date"] = tmdb_info["first_air_date"]
        else:
            tmdb_info["release_date"] = "2004-08-12"
        return tmdb_info
    
    def __make_genres(self, tmdb_info: dict) -> dict:
        if len([type(item) for item in tmdb_info["genres"] if type(item) == int]) > 0:
            return tmdb_info
        tmdb_info["genres"] = [genre_data["id"] for genre_data in tmdb_info["genres"] if genre_data.get("id", None) is not None]
        return tmdb_info
    
    def __make_tmdb_id(self, tmdb_info: dict) -> dict:
        tmdb_info["tmdb_id"] = tmdb_info["id"]
        return tmdb_info

    def __make_last_episode_to_air(self, tmdb_info:dict) -> dict:
        if tmdb_info.get("last_episode_to_air", None) is None:
            tmdb_info["last_episode_to_air"] = None
            return tmdb_info
        else:
            ep_info = tmdb_info["last_episode_to_air"]
            tmdb_info["last_episode_to_air"] = {
                "air_date" : ep_info["air_date"],
                "episode_number": ep_info["episode_number"],
                "season_number": ep_info["season_number"]
            }
            return tmdb_info
        
    def __make_next_episode_to_air(self, tmdb_info:dict) -> dict:
        if tmdb_info.get("next_episode_to_air", None) is None:
            tmdb_info["next_episode_to_air"] = None
            return tmdb_info
        else:
            ep_info = tmdb_info["next_episode_to_air"]
            tmdb_info["next_episode_to_air"] = {
                "air_date" : ep_info["air_date"],
                "episode_number": ep_info["episode_number"],
                "season_number": ep_info["season_number"]
            }
            return tmdb_info
        
    def __make_title(self, tmdb_info:dict)-> dict:
        if tmdb_info.get("title", None) is not None:
            return tmdb_info
        elif tmdb_info.get("name", None) is not None:
            tmdb_info["title"] = tmdb_info["name"]
            return tmdb_info
        else:
            raise MalformedTMDBInfo
        
    def __formatTV_info(self, info:dict) -> dict:
        info["media_type"] = 3
        info = self.__make_seasons(info)
        info = self.__make_alter_titles(info)
        info = self.__make_genres(info)
        info = self.__make_last_episode_to_air(info)
        info = self.__make_release_date(info)
        info = self.__make_title(info)
        info = self.__make_tmdb_id(info)
        info = self.__make_next_episode_to_air(info)
        if info.get("status", None) is None:
            info["status"] = "Ended"
        return info
    
    def __formatMovieInfo(self, info: dict) -> dict:
        info["media_type"] = 1
        info = self.__make_alter_titles(info)
        print(info.get("other_titles", None))
        info = self.__make_genres(info)
        info = self.__make_last_episode_to_air(info)
        info = self.__make_release_date(info)
        info = self.__make_title(info)
        info = self.__make_tmdb_id(info)
        info = self.__make_next_episode_to_air(info)
        info["seasons"] = None
        if info.get("status", None) is None:
            info["status"] = "Ended"
        return info

    def __get_tmdb_info_tv(self, id, append_to_response:str | None = "seasons,translations") -> dict:
        return tmdb.tv.TV(id).info(append_to_response=append_to_response)
                                   
    def __get_tmdb_info_movie(self, id, append_to_response:str | None = "translations") -> dict:
        return tmdb.movies.Movies(id).info(append_to_response=append_to_response)
                                   
    def __getTVInfo(self, id: int) -> dict:
        if not self.tvIdExist(id):
            raise IdDoesNotExist(f"method getTVInfo: {id} does not exist")
        info = self.__get_tmdb_info_tv(id)
        info["info_date"] = get_date(format="%Y-%m-%d")
        info["id"] = id
        info = self.__formatTV_info(info)
        self.__store_tmdb_tv_info(info)
        return info
    
    def __getMovieInfo(self, id: int) -> dict:
        if not self.movieIdExist(id):
            raise IdDoesNotExist(f"method getMovieInfo: {id} does not exist")
        info = self.__get_tmdb_info_movie(id)
        info["info_date"] = get_date(format="%Y-%m-%d")
        info["id"] = id
        info = self.__formatMovieInfo(info)
        self.__store_tmdb_movie_info(info)
        return info

    def get(self, id:int, media_type:int) -> dict:
        #print(id, media_type)
        if not isinstance(id, int):
            raise ValueError("method get: id must be int")
        if media_type == 1:
            return ProviderCommon.make_result(**self.__getMovieInfo(id))
        if media_type == 3:
            return ProviderCommon.make_result(**self.__getTVInfo(id))
        
    def find(self, title:int, media_type:int) -> dict:
        if not isinstance(title, str):
            raise ValueError("method get: tile must be str")
        result = []
        if media_type == 1:
            for ids in self.findMovieByTitle(title=title):
                info = self.__get_info_from_cache_movie(ids)
                if info is None:
                    result.append(ProviderCommon.make_result(**self.__getMovieInfo(ids)))
                else:
                    result.append(ProviderCommon.make_result(**info))                  
        elif media_type == 3:
            for ids in self.findTVByTitle(title=title):
                info = self.__get_info_from_cache_tv(ids)
                if info is None:
                    result.append(ProviderCommon.make_result(**self.__getTVInfo(ids)))
                else:
                    result.append(ProviderCommon.make_result(**info))

        return result
    
    def bestMatchTV(self, name:str) -> str:
        r = process.extractOne(name, [self.IDS_TV[tv] for tv in self.IDS_TV])
        if isinstance(r, tuple):
            name, score = r
            return r
        return None
        

    
        
        
PROVIDERS_LIST["tmdb"] = TMDB_manipulator




from setuptools import setup, find_packages

VERSION = '0.0.4.1'
DESCRIPTION = 'Database to manage media'
LONG_DESCRIPTION = "Ensure your media are here, no double, detect missing media, and can accept any metadata providers, sources"

setup(name="mediaDB",
      version=VERSION,
      author="Benjamin Roget",
      author_email="benjamin.rogetpro@gmail.com",
      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION,
      packages=find_packages(),
      install_requires=[],
      keywords=["python", "media", "manage"],
      classifiers= [
            "Development Status :: 1 - Planning",
            "Intended Audience :: Education",
            "Programming Language :: Python :: 3",
            "Operating System :: POSIX :: Linux ",
            "Framework :: Flask ",
            "License :: Free For Home Use "
        ])



from mediaDB.common import *
from mediaDB.settings import *


class IndexerCommon():

    # VARIABLES
    SETTING_DIRECTORY = os.path.join(CONF_DIR, "Indexers")
    VAR_DIRECTORY = os.path.join(VAR_DIR, "Indexers")
    PROXY = FlareSolverrProxy

    # CREATING FILES & DIRECTORIES
    os.makedirs(SETTING_DIRECTORY, exist_ok=True)
    os.makedirs(VAR_DIRECTORY, exist_ok=True)

    wanted_nfo_specification = ["format", "codec id", "duration", "width",
                                "height", "language", "resolution", "hauteur", "largeur", "duree"]
    wanted_nfo_title = ["text", "video", "audio", "mkv"]

    def make_result(file_title:str, link: str, media_type: int,  seed:int|None=None, codec_id_video:str|None = None,\
                    duration:int|None=None, height:int|None=None, width:int|None=None,\
                         languages:list|None=[], codec_id_audio:str|None=None ) -> dict:
        ...


from os.path import isfile
import requests as r
import bs4



from mediaDB.extension.Indexers.common import IndexerCommon
from mediaDB.common import *
from mediaDB.settings import *
from mediaDB.exceptions import *


class Yggtorrent_manipulator():
    # CONST
        __DATE = datetime.now().strftime("%m_%d_%Y")
        NAME = "YggTorrent"
        CONFIG_EXEMPLE_URL = "https://raw.githubusercontent.com/Strange500/mediaDB/main/exemples/YggTorrent"
        SETTING_FILE = os.path.join(IndexerCommon.SETTING_DIRECTORY, NAME)
        VAR_DIRECTORY = os.path.join(IndexerCommon.VAR_DIRECTORY, NAME)
        CACHE_DIRECTORY = os.path.join(VAR_DIRECTORY, "cache")
        CACHE_DB_TV_FILE = os.path.join(CACHE_DIRECTORY, "DB_tv.json")
        CACHE_DB_MOVIE_FILE = os.path.join(CACHE_DIRECTORY, "DB_movie.json")
        media_types = [1, 3]

        # CREATE NEEDED FILES & DIRECTORY
        os.makedirs(VAR_DIRECTORY, exist_ok=True)
        os.makedirs(CACHE_DIRECTORY, exist_ok=True)
                # Download TMDB config file if not created
        if not isfile(SETTING_FILE) and not wget(CONFIG_EXEMPLE_URL, SETTING_FILE):
            raise ProviderConfigError
        if not isfile(CACHE_DB_TV_FILE) :
            with open(CACHE_DB_TV_FILE, "w", encoding="utf-8") as f:
                dump({}, f, indent=5)
        if not isfile(CACHE_DB_MOVIE_FILE) :
            with open(CACHE_DB_MOVIE_FILE, "w", encoding="utf-8") as f:
                dump({}, f, indent=5)
        # SETTING UP 
        CONFIG = parseConfig(SETTING_FILE)
        if not IndexerCommon.checkConfig(CONFIG, {"api_key": 1, "timeout": 1}):
            raise ProviderConfigError
        API_KEY = CONFIG["api_key"]
        API_KEY = API_KEY[0]
        REQUESTS_TIMEOUT = int(CONFIG["timeout"][0])
        bar(0.001)
            # Download tmdb ids file
        if not isfile(GENRE_MOVIE_FILE):
            movie_list = tmdb.Genres().movie_list()
            with open(GENRE_MOVIE_FILE, "w") as f:
                save_json(f, movie_list)
        bar(0.03)
        if not isfile(GENRE_TV_FILE):
            tv_list = tmdb.Genres().tv_list()
            with open(GENRE_TV_FILE, "w") as f:
                save_json(f, tv_list)
        bar(0.03)
        with open(GENRE_MOVIE_FILE, "r") as f:
            MOVIE_GENRE_IDS = load(f)
        with open(GENRE_TV_FILE, "r") as f:
            TV_GENRE_IDS = load(f)
        bar(0.001)
            # update ids files
        if not isfile(IDS_MOVIE_FILE) :

            if wget(IDS_MOVIES_URL, IDS_MOVIE_FILE+".gz"):
                gzExtract(IDS_MOVIE_FILE+".gz", IDS_MOVIE_FILE) 
                makeIdsFile(IDS_MOVIE_FILE)
        bar(0.59)
        if not isfile(IDS_TV_FILE) :
            if wget(IDS_TV_URL, IDS_TV_FILE+".gz"):
                gzExtract(IDS_TV_FILE+".gz", IDS_TV_FILE)    
                makeIdsFile(IDS_TV_FILE)
        bar(0.2)
            # loads ids
        with open(IDS_MOVIE_FILE, "r", encoding="utf-8") as f:
            IDS_MOVIE = load(f)
        bar(0.05)
        with open(IDS_TV_FILE, "r", encoding="utf-8") as f:
            IDS_TV = load(f)
        bar(0.009)
            # loads DB
        with open(CACHE_DB_TV_FILE, "r", encoding="utf-8") as f:
            CACHE_DB_TV = load(f)
        with open(CACHE_DB_MOVIE_FILE, "r", encoding="utf-8") as f:
            CACHE_DB_MOVIE = load(f)
        bar(1.)
